import "../chunks/chunk-U5RRZUYZ.js";

// src/core/Transaction.ts
import { Blob } from "@alinea/iso";
import { Entry } from "./Entry.js";
import { EntryStatus } from "./EntryRow.js";
import { HttpError } from "./HttpError.js";
import { createId } from "./Id.js";
import { MutationType } from "./Mutation.js";
import { Root } from "./Root.js";
import { Schema } from "./Schema.js";
import { getScope } from "./Scope.js";
import { Type } from "./Type.js";
import { Workspace } from "./Workspace.js";
import { isImage } from "./media/IsImage.js";
import { createFileHash } from "./util/ContentHash.js";
import {
  entryChildrenDir,
  entryFileName,
  entryFilepath,
  entryUrl,
  workspaceMediaDir
} from "./util/EntryFilenames.js";
import { createEntryRow, entryParentPaths } from "./util/EntryRows.js";
import { generateKeyBetween } from "./util/FractionalIndexing.js";
import { basename, extname, join, normalize } from "./util/Paths.js";
import { slugify } from "./util/Slugs.js";
var Operation = class _Operation {
  static Data = Symbol.for("@alinea/Operation.Data");
  constructor(tx) {
    this[_Operation.Data] = tx;
  }
  typeName(config, type) {
    const typeNames = Schema.typeNames(config.schema);
    const typeName = typeNames.get(type);
    if (!typeName)
      throw new Error(`Type "${Type.label(type)}" not found in Schema`);
    return typeName;
  }
};
var UploadOperation = class extends Operation {
  id = createId();
  constructor(query) {
    super(async (cms) => {
      const entryId = this.id;
      const { file, createPreview } = query;
      const { workspace: _workspace, root: _root, parentId: _parentId } = query;
      const { config, connect } = cms;
      const fileName = Array.isArray(file) ? file[0] : file.name;
      const body = Array.isArray(file) ? file[1] : await file.arrayBuffer();
      const workspace = _workspace ?? Object.keys(config.workspaces)[0];
      const root = _root ?? Workspace.defaultMediaRoot(config.workspaces[workspace]);
      const extension = extname(fileName);
      const path = slugify(basename(fileName, extension));
      const directory = workspaceMediaDir(config, workspace);
      const uploadLocation = join(directory, path + extension);
      const cnx = await connect();
      const info = await cnx.prepareUpload(uploadLocation);
      const previewData = isImage(fileName) ? await createPreview?.(file instanceof Blob ? file : new Blob([body])) : void 0;
      await fetch(info.url, { method: info.method ?? "POST", body }).then(
        async (result) => {
          if (!result.ok)
            throw new HttpError(
              result.status,
              `Could not reach server for upload`
            );
        }
      );
      const parent = _parentId ? await cms.first({
        select: Entry,
        id: _parentId,
        status: "preferPublished"
      }) : null;
      const title = basename(fileName, extension);
      const hash = await createFileHash(new Uint8Array(body));
      const { mediaDir } = Workspace.data(config.workspaces[workspace]);
      const prefix = mediaDir && normalize(mediaDir);
      const fileLocation = prefix && info.location.startsWith(prefix) ? info.location.slice(prefix.length) : info.location;
      const entryData = {
        title,
        location: fileLocation,
        extension,
        size: body.byteLength,
        hash,
        ...previewData
      };
      const entry = await createEntry(
        config,
        "MediaFile",
        {
          path,
          id: entryId,
          workspace,
          root,
          data: entryData
        },
        parent
      );
      const parentPaths = entryParentPaths(config, entry);
      const entryFile = entryFileName(config, entry, parentPaths);
      return [
        {
          type: MutationType.Upload,
          entryId,
          url: info.previewUrl,
          file: info.location
        },
        {
          type: MutationType.Create,
          locale: entry.locale,
          entryId: entry.id,
          file: entryFile,
          entry
        }
      ];
    });
  }
};
var DeleteOp = class extends Operation {
  constructor(entryIds) {
    super(async (cms) => {
      const entries = await cms.find({
        select: Entry,
        id: { in: entryIds },
        status: "preferPublished"
      });
      return entries.map((entry) => {
        const parentPaths = entryParentPaths(cms.config, entry);
        const file = entryFileName(cms.config, entry, parentPaths);
        return {
          type: MutationType.Remove,
          entryId: entry.id,
          locale: entry.locale,
          file
        };
      });
    });
    this.entryIds = entryIds;
  }
};
var UpdateOperation = class extends Operation {
  constructor(query) {
    super(async (cms) => {
      const { status: changeStatus, set } = query;
      const entryId = query.id;
      let status;
      if (changeStatus === EntryStatus.Draft)
        status = "preferDraft";
      else if (changeStatus === EntryStatus.Archived)
        status = "preferPublished";
      else if (changeStatus === EntryStatus.Published)
        status = "preferDraft";
      else
        status = "preferPublished";
      const entry = await cms.get({
        select: Entry,
        id: entryId,
        status
      });
      const parent = entry.parentId ? await cms.get({
        select: Entry,
        id: entry.parentId,
        status: "preferPublished"
      }) : void 0;
      const parentPaths = entryParentPaths(cms.config, entry);
      const file = entryFileName(
        cms.config,
        { ...entry, status: entry.status },
        parentPaths
      );
      const type = cms.config.schema[entry.type];
      const mutations = [];
      const createDraft = changeStatus === EntryStatus.Draft;
      if (createDraft)
        mutations.push({
          type: MutationType.Edit,
          entryId,
          locale: entry.locale,
          file,
          entry: await createEntry(
            cms.config,
            this.typeName(cms.config, type),
            {
              ...entry,
              status: EntryStatus.Draft,
              data: { ...entry.data, ...set }
            },
            parent
          )
        });
      else if (set)
        mutations.push({
          type: MutationType.Patch,
          entryId,
          locale: entry.locale,
          file,
          patch: set
        });
      switch (changeStatus) {
        case EntryStatus.Published:
          mutations.push({
            type: MutationType.Publish,
            locale: entry.locale,
            status: entry.status,
            entryId,
            file
          });
          break;
        case EntryStatus.Archived:
          mutations.push({
            type: MutationType.Archive,
            entryId,
            locale: entry.locale,
            file
          });
          break;
      }
      return mutations;
    });
  }
};
var CreateOperation = class extends Operation {
  id = createId();
  constructor(query) {
    super(async (cms) => {
      const entryId = this.id;
      const entry = await entryRow();
      const parentPaths = entryParentPaths(cms.config, entry);
      const file = entryFileName(cms.config, entry, parentPaths);
      return [
        {
          type: MutationType.Create,
          entryId: entry.id,
          locale: entry.locale,
          file,
          entry
        }
      ];
      async function entryRow() {
        const { workspace, root, parentId, locale, set } = query;
        const typeName = getScope(cms.config).nameOf(query.type);
        if (!typeName)
          throw new Error(
            `Type "${Type.label(query.type)}" not found in Schema`
          );
        const partial = {
          id: entryId,
          type: typeName,
          workspace,
          root,
          parentId,
          locale,
          data: set
        };
        let parent;
        try {
          parent = await (parentId ? cms.get({
            select: Entry,
            id: parentId,
            status: "preferPublished"
          }) : void 0);
        } catch {
          throw new Error(
            `Parent entry not found: ${parentId}, try commiting it first?`
          );
        }
        const previousIndex = await cms.first({
          status: "preferPublished",
          select: Entry.index,
          parentId: parent?.id ?? null,
          orderBy: { desc: Entry.index, caseSensitive: true }
        });
        const index = generateKeyBetween(previousIndex, null);
        return createEntry(cms.config, typeName, { ...partial, index }, parent);
      }
    });
  }
};
async function createEntry(config, typeName, partial = { title: "Entry" }, parent) {
  const type = config.schema[typeName];
  const workspace = parent?.workspace ?? partial.workspace ?? Object.keys(config.workspaces)[0];
  const root = parent?.root ?? partial.root ?? Object.keys(config.workspaces[workspace])[0];
  const locale = parent?.locale ?? partial.locale ?? Root.defaultLocale(config.workspaces[workspace][root]) ?? null;
  const title = partial.data?.title ?? partial.title ?? "Entry";
  const status = partial.status ?? EntryStatus.Published;
  const path = slugify(
    status === EntryStatus.Published && partial.data?.path || (partial.path ?? title)
  );
  const entryData = { title, path, ...partial.data };
  const id = partial.id ?? createId();
  const details = {
    id,
    status,
    type: typeName,
    title,
    path,
    seeded: null,
    workspace,
    root,
    level: parent ? parent.level + 1 : 0,
    parentId: parent?.id ?? null,
    locale,
    index: partial.index ?? "a0",
    modifiedAt: 0,
    active: partial.active ?? true,
    main: partial.main ?? true,
    data: entryData,
    searchableText: Type.searchableText(type, entryData)
  };
  const isI18n = Boolean(Root.data(config.workspaces[workspace][root]).i18n);
  const parentPaths = parent?.childrenDir.split("/").filter(Boolean).slice(isI18n ? 1 : 0) ?? [];
  const filePath = entryFilepath(config, details, parentPaths);
  const childrenDir = entryChildrenDir(config, details, parentPaths);
  const urlMeta = {
    locale,
    path,
    status,
    parentPaths
  };
  const url = entryUrl(type, urlMeta);
  return createEntryRow(config, {
    ...details,
    filePath,
    childrenDir,
    parentDir: childrenDir.split("/").slice(0, -1).join("/"),
    url
  });
}
export {
  CreateOperation,
  DeleteOp,
  Operation,
  UpdateOperation,
  UploadOperation
};
